clear all
clc

% x = [X U]', where X = [mole fractions (1 to 41) ; holdups (42 to 82)]
% U = [LT VB D B F zF qF]

%Disturbances: p_v, F, zF and qF

p_v = 0.018;     % 0.01 - 0.02 mol/s

prices = [p_v, 1, 1, 2];

F = 0.7 ;       % 1.0 - 1.6 mol/s
zF = 0.5;
qF = 1;

d = [F zF qF]';

%Inequality constraints (upper/lower bounds):
xB_max = 0.01;          % (1) x_heavy >= 0.99 --> x_light <= 0.01. xB  = X(1)
xD_min = 0.95;          % (2) x_light >= 0.95. xD = X(41)
Vmax = 4.008;           % (3)

%Upper and lower bounds:
lb = zeros(89,1);      % cannot have negative values
lb(41) = xD_min;        % Ineq. constraint 1. xD = X(41)
lb(83) = 0.1;
lb(84) = 0.1;

ub = ones(89,1);        %no fractions can be larger than 1.

ub(1) = xB_max;         % Ineq. constraint 2. xB = X(1)

i=42:82;                
ub(i) = inf;            %maximum holdups

%i=83:89;
%ub(i) = 10;             %upper bounds for the U's

%i=84;
%ub(i) = Vmax;           % Ineq. constraint 3: vapor streams cannot be larger than Vmax

ub(83) = Vmax;           % Ineq. constraint 3: vapor streams cannot be larger than Vmax
ub(84) = Vmax;
ub(85) = F;
ub(86) = F;
ub(87:89) = 10;


%Controller:
par.KcB = 10;
par.KcD = 10;
par.MDs = 0.5;
par.MBs = 0.5;
par.Ds = 0.5;
par.Bs = 0.5;

%initial guess
x0 = 0.5*ones(89,1);   
x0(1:41) = linspace(0.01,0.95,41);
x0(87)=F; x0(88)=zF; x0(89)=qF;

options = optimset('TolCon', 1e-8,'TolFun', 1e-8, 'TolX', 1e-8,'Algorithm','active-set'); %'trust-region-reflective' (default),'active-set','interior-point','sqp'
[x,fval,Eflag] = fmincon(@(x)objfunc(x,prices),x0,[],[],[],[],lb,ub,@(x)nonlinconstr(x,d,par),options);
Eflag


disp('Fraction of light component on each stage:')
disp(x(1:41))

disp('Holdups:')
disp(x(42:82))


disp('LT')
disp(x(83))

disp('VB')
disp(x(84))

disp('D')
disp(x(85))

disp('B')
disp(x(86))

disp('xB')
disp(1-x(1))

disp('xD')
disp(x(41))

disp('J')
disp(fval)

disp('Eflag')
disp(Eflag)
